from setuptools import setup

setup(
    name="pymold",
    version="0.1.0.2",
    description="A simple, fast template engine for Python.",
    long_description="https://github.com/Bogdanp/mold",
    packages=[
        "mold",
    ],

    install_requires=[
    ],

    author="Bogdan Popa",
    author_email="popa.bogdanp@gmail.com",
    url="https://github.com/Bogdanp/mold",
    keywords=["web"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
    ]
)
